@@ -134,7 +134,7 @@ extern unsigned int verbose;
 #define mxfind2(it, value, cont) ((it = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
 #define map_has_key(m, k)        ((m).end() != (m).find(k))
 
-void mtx_common_init(std::string const &program_name);
+void mtx_common_init(std::string const &program_name, char const *argv0);
 std::string const &get_program_name();
 
 #endif // MTX_COMMON_COMMON_H
