@@ -16,8 +16,6 @@
 #include "common/common_pch.h"
 
 int64_t get_current_time_millis();
-std::string get_application_data_folder();
-std::string get_installation_path();
 
 #if defined(SYS_WINDOWS)
 
@@ -42,6 +40,9 @@ unsigned int get_windows_version();
 namespace mtx {
 
 int system(std::string const &command);
+void determine_path_to_current_executable(std::string const &argv0);
+bfs::path get_application_data_folder();
+bfs::path const &get_installation_path();
 
 }
 
