@@ -263,7 +263,7 @@ protected:
 
   bool m_ignore_nalu_size_length_errors, m_discard_actual_frames;
 
-  debugging_option_c m_debug_keyframe_detection, m_debug_nalu_types, m_debug_timecodes, m_debug_sps_info;
+  debugging_option_c m_debug_keyframe_detection, m_debug_nalu_types, m_debug_timecodes, m_debug_sps_info, m_debug_trailing_zero_byte_removal;
   std::map<int, std::string> m_nalu_names_by_type;
 
   struct stats_t {
@@ -415,6 +415,7 @@ protected:
   void flush_unhandled_nalus();
   void write_nalu_size(unsigned char *buffer, size_t size, int this_nalu_size_length = -1) const;
   memory_cptr create_nalu_with_size(const memory_cptr &src, bool add_extra_data = false);
+  void remove_trailing_zero_bytes(memory_c &memory);
   void init_nalu_names();
 };
 typedef std::shared_ptr<avc_es_parser_c> avc_es_parser_cptr;
