@@ -18,6 +18,7 @@
 
 #define Q(s)  to_qs(s)
 #define QY(s) to_qs(Y(s))
+#define QNY(singular, plural, count) to_qs(NY(singular, plural, count))
 
 inline QString
 to_qs(std::string const &source) {
@@ -29,6 +30,11 @@ to_qs(std::wstring const &source) {
   return QString::fromStdWString(source);
 }
 
+inline QString
+to_qs(boost::format const &source) {
+  return QString::fromUtf8(source.str().c_str());
+}
+
 inline std::string
 to_utf8(QString const &source) {
   return std::string{ source.toUtf8().data() };
