@@ -57,11 +57,12 @@ create_node_name(const char *name,
 
 document_cptr
 load_file(std::string const &file_name,
-          unsigned int options) {
+          unsigned int options,
+          boost::optional<int64_t> max_read_size) {
   auto af_in = mm_file_io_c::open(file_name, MODE_READ);
   mm_text_io_c in(af_in.get(), false);
   std::string content;
-  auto bytes_to_read = in.get_size() - in.get_byte_order_length();
+  auto bytes_to_read = (max_read_size ? std::min(in.get_size(), *max_read_size) : in.get_size()) - in.get_byte_order_length();
 
   if (in.read(content, bytes_to_read) != bytes_to_read)
     throw mtx::mm_io::end_of_file_x{};
