@@ -22,8 +22,7 @@ class xtr_pgs_c: public xtr_base_c {
 public:
   xtr_pgs_c(const std::string &codec_id, int64_t tid, track_spec_t &tspec);
 
-  virtual void handle_frame(memory_cptr &frame, KaxBlockAdditions *additions, int64_t timecode, int64_t duration, int64_t bref, int64_t fref,
-                            bool keyframe, bool discardable, bool references_valid);
+  virtual void handle_frame(xtr_frame_t &f);
 
   virtual const char *get_container_name() {
     return "SUP";
