@@ -72,6 +72,7 @@
 #include "common/command_line.h"
 #include "common/ebml.h"
 #include "common/endian.h"
+#include "common/hevc.h"
 #include "common/kax_file.h"
 #include "common/mm_io.h"
 #include "common/mm_io_x.h"
@@ -359,6 +360,15 @@ create_codec_dependent_private_info(KaxCodecPrivate &c_priv,
                : avcc.m_profile_idc == 244 ? "High 4:4:4 Predictive"
                :                             Y("Unknown"))
             % (avcc.m_level_idc / 10) % (avcc.m_level_idc % 10)).str();
+  } else if ((codec_id == MKV_V_MPEGH_HEVC) && ('v' == track_type) && (c_priv.GetSize() >= 4)) {
+    auto hevcc = hevc::hevcc_c::unpack(std::make_shared<memory_c>(c_priv.GetBuffer(), c_priv.GetSize(), false));
+
+    return (boost::format(Y(" (HEVC profile: %1% @L%2%.%3%)"))
+            % (  hevcc.m_general_profile_idc == 1 ? "Main"
+               : hevcc.m_general_profile_idc == 2 ? "Main 10"
+               : hevcc.m_general_profile_idc == 3 ? "Main Still Picture"
+               :                                    Y("Unknown"))
+            % (hevcc.m_general_level_idc / 3 / 10) % (hevcc.m_general_level_idc / 3 % 10)).str();
   }
 
   return "";
@@ -1719,8 +1729,9 @@ process_file(const std::string &file_name) {
 }
 
 void
-setup(const std::string &locale) {
-  mtx_common_init("mkvinfo");
+setup(char const *argv0,
+      std::string const &locale) {
+  mtx_common_init("mkvinfo", argv0);
 
   init_locales(locale);
   init_common_boost_formats();
@@ -1741,14 +1752,16 @@ console_main() {
 int
 main(int argc,
      char **argv) {
-  setup();
+  setup(argv[0]);
 
   g_options = info_cli_parser_c(command_line_utf8(argc, argv)).run();
 
   init_common_boost_formats();
 
   if (g_options.m_use_gui)
-    return ui_run(argc, argv);
+    ui_run(argc, argv);
   else
-    return console_main();
+    console_main();
+
+  mxexit(0);
 }
