@@ -24,7 +24,7 @@ extern options_c g_options;
 
 int console_main();
 bool process_file(const std::string &file_name);
-void setup(const std::string &locale = "");
+void setup(char const *argv0, const std::string &locale = "");
 void cleanup();
 
 std::string create_element_text(const std::string &text, int64_t position, int64_t size);
