@@ -67,6 +67,8 @@
 #if defined(HAVE_FLAC_FORMAT_H)
 # include "output/p_flac.h"
 #endif
+#include "output/p_hevc.h"
+#include "output/p_hevc_es.h"
 #include "output/p_kate.h"
 #include "output/p_mp3.h"
 #include "output/p_mpeg1_2.h"
@@ -171,6 +173,30 @@ kax_track_t::fix_display_dimension_parameters() {
   }
 }
 
+void
+kax_track_t::add_track_tags_to_identification(std::vector<std::string> &verbose_info) {
+  if (!tags)
+    return;
+
+  for (auto const &tag_elt : *tags) {
+    auto tag = dynamic_cast<KaxTag *>(tag_elt);
+    if (!tag)
+      continue;
+
+    for (auto const &simple_tag_elt : *tag) {
+      auto simple_tag = dynamic_cast<KaxTagSimple *>(simple_tag_elt);
+      if (!simple_tag)
+        continue;
+
+      auto name  = mtx::tags::get_simple_name(*simple_tag);
+      auto value = mtx::tags::get_simple_value(*simple_tag);
+
+      if (!name.empty())
+        verbose_info.emplace_back((boost::format("tag_%1%:%2%") % balg::to_lower_copy(name) % escape(value)).str());
+    }
+  }
+}
+
 /*
    Probes a file by simply comparing the first four bytes to the EBML
    head signature.
@@ -636,8 +662,7 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
     ++m_attachment_id;
     attach_mode_e attach_mode;
-    if (   !matt.id
-        || !matt.data->get_size()
+    if (   !matt.data->get_size()
         || matt.mime_type.empty()
         || matt.name.empty()
         || ((attach_mode = attachment_requested(m_attachment_id)) == ATTACH_MODE_SKIP))
@@ -1326,6 +1351,13 @@ kax_reader_c::create_video_packetizer(kax_track_t *t,
     set_track_packetizer(t, new mpeg1_2_video_packetizer_c(this, nti, version, t->v_frate, t->v_width, t->v_height, t->v_dwidth, t->v_dheight, true));
     show_packetizer_info(t->tnum, t->ptzr_ptr);
 
+  } else if (t->codec.is(CT_V_MPEGH_P2)) {
+    if (t->ms_compat)
+      create_hevc_es_video_packetizer(t, nti);
+    else
+      create_hevc_video_packetizer(t, nti);
+    show_packetizer_info(t->tnum, t->ptzr_ptr);
+
   } else if (t->codec.is(CT_V_MPEG4_P2)) {
     bool is_native = IS_MPEG4_L2_CODECID(t->codec_id);
     set_track_packetizer(t, new mpeg4_p2_video_packetizer_c(this, nti, t->v_frate, t->v_width, t->v_height, is_native));
@@ -1466,6 +1498,30 @@ kax_reader_c::create_flac_audio_packetizer(kax_track_t *t,
 
 #endif  // HAVE_FLAC_FORMAT_H
 
+void
+kax_reader_c::create_hevc_es_video_packetizer(kax_track_t *t,
+                                              track_info_c &nti) {
+  hevc_es_video_packetizer_c *ptzr = new hevc_es_video_packetizer_c(this, nti);
+  set_track_packetizer(t, ptzr);
+
+  ptzr->set_video_pixel_dimensions(t->v_width, t->v_height);
+
+  show_packetizer_info(t->tnum, t->ptzr_ptr);
+}
+
+void
+kax_reader_c::create_hevc_video_packetizer(kax_track_t *t,
+                                           track_info_c &nti) {
+  if (!nti.m_private_data || !nti.m_private_data->get_size()) {
+    // avc_es_parser_cptr parser = parse_first_hevc_frame(t, nti);
+    create_hevc_es_video_packetizer(t, nti);
+    return;
+  }
+
+  set_track_packetizer(t, new hevc_video_packetizer_c(this, nti, t->v_frate, t->v_width, t->v_height));
+  show_packetizer_info(t->tnum, t->ptzr_ptr);
+}
+
 void
 kax_reader_c::create_mp3_audio_packetizer(kax_track_t *t,
                                           track_info_c &nti) {
@@ -2185,6 +2241,8 @@ kax_reader_c::identify() {
 
     if (track->codec.is(CT_V_MPEG4_P10))
       verbose_info.push_back(track->ms_compat ? "packetizer:mpeg4_p10_es_video" : "packetizer:mpeg4_p10_video");
+    else if (track->codec.is(CT_V_MPEGH_P2))
+      verbose_info.push_back(track->ms_compat ? "packetizer:mpegh_p2_es_video"  : "packetizer:mpegh_p2_video");
 
     if (0 != track->default_duration)
       verbose_info.push_back((boost::format("default_duration:%1%") % track->default_duration).str());
@@ -2209,6 +2267,8 @@ kax_reader_c::identify() {
     else
       info = track->codec_id;
 
+    track->add_track_tags_to_identification(verbose_info);
+
     id_result_track(track->tnum,
                       track->type == 'v' ? ID_RESULT_TRACK_VIDEO
                     : track->type == 'a' ? ID_RESULT_TRACK_AUDIO
@@ -2219,17 +2279,17 @@ kax_reader_c::identify() {
   }
 
   for (auto &attachment : g_attachments)
-    id_result_attachment(attachment.ui_id, attachment.mime_type, attachment.data->get_size(), attachment.name, attachment.description);
+    id_result_attachment(attachment.ui_id, attachment.mime_type, attachment.data->get_size(), attachment.name, attachment.description, attachment.id);
 
   if (m_chapters)
     id_result_chapters(count_chapter_atoms(*m_chapters));
 
   if (m_tags)
-    id_result_tags(ID_RESULT_GLOBAL_TAGS_ID, count_simple_tags(*m_tags));
+    id_result_tags(ID_RESULT_GLOBAL_TAGS_ID, mtx::tags::count_simple(*m_tags));
 
   for (auto &track : m_tracks)
     if (track->ok && track->tags)
-      id_result_tags(track->tnum, count_simple_tags(*track->tags));
+      id_result_tags(track->tnum, mtx::tags::count_simple(*track->tags));
 }
 
 void
