@@ -17,6 +17,8 @@
 #include "common/common_pch.h"
 
 #include "common/bit_cursor.h"
+#include "common/codec.h"
+#include "common/debugging.h"
 #include "common/dts.h"
 #include "common/mm_multi_file_io.h"
 #include "common/mpeg1_2.h"
@@ -108,7 +110,7 @@ struct mpeg_ps_track_t {
   char type;                    // 'v' for video, 'a' for audio, 's' for subs
   mpeg_ps_id_t id;
   int sort_key;
-  uint32_t fourcc;
+  codec_c codec;
 
   bool provide_timecodes;
   int64_t timecode_offset, timecode_b_frame_offset;
@@ -131,7 +133,6 @@ struct mpeg_ps_track_t {
     ptzr(-1),
     type(0),
     sort_key(0),
-    fourcc(0),
     provide_timecodes(false),
     timecode_offset(std::numeric_limits<int64_t>::max()),
     timecode_b_frame_offset{0},
@@ -200,7 +201,7 @@ private:
   std::vector<mpeg_ps_track_ptr> tracks;
   std::map<generic_packetizer_c *, mpeg_ps_track_ptr> m_ptzr_to_track_map;
 
-  bool m_debug_timecodes;
+  debugging_option_c m_debug_timecodes;
 
 public:
   mpeg_ps_reader_c(const track_info_c &ti, const mm_io_cptr &in);
