@@ -353,6 +353,7 @@ public:
 
   void handle_timecode_wrap(timecode_c &pts, timecode_c &dts);
   bool detect_timecode_wrap(timecode_c &timecode);
+  void adjust_timecode_for_wrap(timecode_c &timecode);
 };
 
 typedef std::shared_ptr<mpeg_ts_track_c> mpeg_ts_track_ptr;
@@ -362,7 +363,7 @@ protected:
   bool PAT_found, PMT_found;
   int16_t PMT_pid;
   int es_to_process;
-  timecode_c m_global_timecode_offset;
+  timecode_c m_global_timecode_offset, m_stream_timecode;
 
   mpeg_ts_input_type_e input_status; // can be INPUT_PROBE, INPUT_READ
   int track_buffer_ready;
