@@ -2,7 +2,9 @@
 
 #include "common/qt.h"
 #include "mkvtoolnix-gui/forms/main_window.h"
+#include "mkvtoolnix-gui/job_widget/job_widget.h"
 #include "mkvtoolnix-gui/main_window/main_window.h"
+#include "mkvtoolnix-gui/main_window/status_bar_progress_widget.h"
 #include "mkvtoolnix-gui/merge_widget/merge_widget.h"
 #include "mkvtoolnix-gui/util/settings.h"
 #include "mkvtoolnix-gui/util/util.h"
@@ -17,12 +19,17 @@ MainWindow *MainWindow::ms_mainWindow = nullptr;
 MainWindow::MainWindow(QWidget *parent)
   : QMainWindow{parent}
   , ui{new Ui::MainWindow}
+  , m_toolMerge{}
+  , m_toolJobs{}
 {
   ms_mainWindow = this;
 
   // Setup UI controls.
   ui->setupUi(this);
 
+  m_statusBarProgress = new StatusBarProgressWidget{this};
+  ui->statusBar->addPermanentWidget(m_statusBarProgress);
+
   setupToolSelector();
 
   // Setup window properties.
@@ -67,29 +74,36 @@ void
 MainWindow::setupToolSelector() {
   // ui->tool->setIconSize(QSize{48, 48});
 
-  MergeWidget *toolMerge = new MergeWidget{ui->tool};
+  m_toolMerge = new MergeWidget{ui->tool};
+  m_toolJobs  = new JobWidget{ui->tool};
 
-  // ui->tool->insertTab(0, toolMerge,                    QIcon{":/icons/48x48/document-import.png"},       QY("merge"));
-  // ui->tool->insertTab(1, createNotImplementedWidget(), QIcon{":/icons/48x48/document-export.png"},       QY("extract"));
-  // ui->tool->insertTab(2, createNotImplementedWidget(), QIcon{":/icons/48x48/document-edit-verify.png"},  QY("info"));
-  // ui->tool->insertTab(4, createNotImplementedWidget(), QIcon{":/icons/48x48/document-open-recent.png"},  QY("edit chapters"));
-  // ui->tool->insertTab(5, createNotImplementedWidget(), QIcon{":/icons/48x48/document-edit-decrypt.png"}, QY("edit tags"));
-
-  ui->tool->insertTab(0, toolMerge,                    QIcon{":/icons/48x48/merge.png"},                      QY("merge"));
+  ui->tool->insertTab(0, m_toolMerge,                  QIcon{":/icons/48x48/merge.png"},                      QY("merge"));
   ui->tool->insertTab(1, createNotImplementedWidget(), QIcon{":/icons/48x48/split.png"},                      QY("extract"));
   ui->tool->insertTab(2, createNotImplementedWidget(), QIcon{":/icons/48x48/document-preview-archive.png"},   QY("info"));
   ui->tool->insertTab(3, createNotImplementedWidget(), QIcon{":/icons/48x48/document-edit.png"},              QY("edit headers"));
   ui->tool->insertTab(4, createNotImplementedWidget(), QIcon{":/icons/48x48/story-editor.png"},               QY("edit chapters"));
   ui->tool->insertTab(5, createNotImplementedWidget(), QIcon{":/icons/48x48/document-edit-sign-encrypt.png"}, QY("edit tags"));
-  ui->tool->insertTab(6, createNotImplementedWidget(), QIcon{":/icons/48x48/view-task.png"},                  QY("job queue"));
+  ui->tool->insertTab(6, m_toolJobs,                   QIcon{":/icons/48x48/view-task.png"},                  QY("job queue"));
 
   for (int i = 0; 6 >= i; i++)
     ui->tool->setTabEnabled(i, true);
 
   ui->tool->setCurrentIndex(0);
+
+  connect(m_toolJobs->getModel(), SIGNAL(progressChanged(unsigned int,unsigned int)), m_statusBarProgress, SLOT(setProgress(unsigned int,unsigned int)));
 }
 
 MainWindow *
 MainWindow::get() {
   return ms_mainWindow;
 }
+
+MergeWidget *
+MainWindow::getMergeWidget() {
+  return get()->m_toolMerge;
+}
+
+JobWidget *
+MainWindow::getJobWidget() {
+  return get()->m_toolJobs;
+}
