@@ -5,6 +5,7 @@
 #include "common/qt.h"
 #include "common/stereo_mode.h"
 #include "mkvtoolnix-gui/merge_widget/merge_widget.h"
+#include "mkvtoolnix-gui/merge_widget/playlist_scanner.h"
 #include "mkvtoolnix-gui/forms/merge_widget.h"
 #include "mkvtoolnix-gui/util/file_identifier.h"
 #include "mkvtoolnix-gui/util/file_type_filter.h"
@@ -13,6 +14,7 @@
 
 #include <QFileDialog>
 #include <QList>
+#include <QMessageBox>
 #include <QString>
 
 void
@@ -21,27 +23,31 @@ MergeWidget::setupControlLists() {
 
   m_audioControls << ui->trackNameLabel << ui->trackName << ui->trackLanguageLabel << ui->trackLanguage << ui->defaultTrackFlagLabel << ui->defaultTrackFlag << ui->forcedTrackFlagLabel << ui->forcedTrackFlag
                   << ui->compressionLabel << ui->compression << ui->trackTagsLabel << ui->trackTags << ui->browseTrackTags << ui->timecodesAndDefaultDurationBox
-                  << ui->delayLabel << ui->delay << ui->stretchByLabel << ui->stretchBy << ui->timecodesLabel << ui->timecodes << ui->browseTimecodes << ui->audioPropertiesBox << ui->aacIsSBR << ui->cuesLabel << ui->cues;
+                  << ui->delayLabel << ui->delay << ui->stretchByLabel << ui->stretchBy << ui->timecodesLabel << ui->timecodes << ui->browseTimecodes << ui->audioPropertiesBox << ui->aacIsSBR << ui->cuesLabel << ui->cues
+                  << ui->propertiesLabel << ui->generalOptionsBox;
 
   m_videoControls << ui->trackNameLabel << ui->trackName << ui->trackLanguageLabel << ui->trackLanguage << ui->defaultTrackFlagLabel << ui->defaultTrackFlag
                   << ui->forcedTrackFlagLabel << ui->forcedTrackFlag << ui->compressionLabel << ui->compression << ui->trackTagsLabel << ui->trackTags << ui->browseTrackTags << ui->timecodesAndDefaultDurationBox
                   << ui->delayLabel << ui->delay << ui->stretchByLabel << ui->stretchBy << ui->defaultDurationLabel << ui->defaultDuration << ui->timecodesLabel << ui->timecodes << ui->browseTimecodes
                   << ui->picturePropertiesBox << ui->setAspectRatio << ui->aspectRatio << ui->setDisplayWidthHeight << ui->displayWidth << ui->displayDimensionsXLabel << ui->displayHeight << ui->stereoscopyLabel
-                  << ui->stereoscopy << ui->croppingLabel << ui->cropping << ui->cuesLabel << ui->cues;
+                  << ui->stereoscopy << ui->croppingLabel << ui->cropping << ui->cuesLabel << ui->cues
+                  << ui->propertiesLabel << ui->generalOptionsBox << ui->fixBitstreamTimingInfo;
 
   m_subtitleControls << ui->trackNameLabel << ui->trackName << ui->trackLanguageLabel << ui->trackLanguage << ui->defaultTrackFlagLabel << ui->defaultTrackFlag
                      << ui->forcedTrackFlagLabel << ui->forcedTrackFlag << ui->compressionLabel << ui->compression << ui->trackTagsLabel << ui->trackTags << ui->browseTrackTags << ui->timecodesAndDefaultDurationBox
                      << ui->delayLabel << ui->delay << ui->stretchByLabel << ui->stretchBy << ui->timecodesLabel << ui->timecodes << ui->browseTimecodes
-                     << ui->subtitleAndChapterPropertiesBox << ui->characterSetLabel << ui->subtitleCharacterSet << ui->cuesLabel << ui->cues;
+                     << ui->subtitleAndChapterPropertiesBox << ui->characterSetLabel << ui->subtitleCharacterSet << ui->cuesLabel << ui->cues
+                     << ui->propertiesLabel << ui->generalOptionsBox;
 
-  m_chapterControls << ui->subtitleAndChapterPropertiesBox << ui->characterSetLabel << ui->subtitleCharacterSet;
+  m_chapterControls << ui->subtitleAndChapterPropertiesBox << ui->characterSetLabel << ui->subtitleCharacterSet << ui->propertiesLabel << ui->generalOptionsBox;
 
   m_allInputControls << ui->muxThisLabel << ui->muxThis << ui->trackNameLabel << ui->trackName << ui->trackLanguageLabel << ui->trackLanguage << ui->defaultTrackFlagLabel << ui->defaultTrackFlag
                      << ui->forcedTrackFlagLabel << ui->forcedTrackFlag << ui->compressionLabel << ui->compression << ui->trackTagsLabel << ui->trackTags << ui->browseTrackTags << ui->timecodesAndDefaultDurationBox
                      << ui->delayLabel << ui->delay << ui->stretchByLabel << ui->stretchBy << ui->defaultDurationLabel << ui->defaultDuration << ui->timecodesLabel << ui->timecodes << ui->browseTimecodes
                      << ui->picturePropertiesBox << ui->setAspectRatio << ui->aspectRatio << ui->setDisplayWidthHeight << ui->displayWidth << ui->displayDimensionsXLabel << ui->displayHeight << ui->stereoscopyLabel
                      << ui->stereoscopy << ui->croppingLabel << ui->cropping << ui->audioPropertiesBox << ui->aacIsSBR << ui->subtitleAndChapterPropertiesBox << ui->characterSetLabel << ui->subtitleCharacterSet
-                     << ui->miscellaneousBox << ui->cuesLabel << ui->cues << ui->userDefinedTrackOptionsLabel << ui->userDefinedTrackOptions;
+                     << ui->miscellaneousBox << ui->cuesLabel << ui->cues << ui->userDefinedTrackOptionsLabel << ui->userDefinedTrackOptions
+                     << ui->propertiesLabel << ui->generalOptionsBox << ui->fixBitstreamTimingInfo;
 
   m_comboBoxControls << ui->muxThis << ui->trackLanguage << ui->defaultTrackFlag << ui->forcedTrackFlag << ui->compression << ui->cues << ui->stereoscopy << ui->aacIsSBR << ui->subtitleCharacterSet;
 }
@@ -91,6 +97,9 @@ MergeWidget::setupInputControls() {
     for (auto idx = 0; control->count() > idx; ++idx)
       control->setItemData(idx, idx);
 
+  for (auto const &control : m_comboBoxControls)
+    control->view()->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+
   // Connect signals & slots.
   connect(ui->files->selectionModel(),  SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)), this, SLOT(onFileSelectionChanged()));
   connect(ui->tracks->selectionModel(), SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)), this, SLOT(onTrackSelectionChanged()));
@@ -105,7 +114,7 @@ MergeWidget::setupInputControls() {
   connect(m_addFilesAction,           SIGNAL(triggered()), this, SLOT(onAddFiles()));
   connect(m_appendFilesAction,        SIGNAL(triggered()), this, SLOT(onAppendFiles()));
   connect(m_addAdditionalPartsAction, SIGNAL(triggered()), this, SLOT(onAddAdditionalParts()));
-  // connect(m_removeFilesAction,        SIGNAL(triggered()), this, SLOT(onRemoveFiles()));
+  connect(m_removeFilesAction,        SIGNAL(triggered()), this, SLOT(onRemoveFiles()));
   connect(m_removeAllFilesAction,     SIGNAL(triggered()), this, SLOT(onRemoveAllFiles()));
 
   onTrackSelectionChanged();
@@ -118,7 +127,7 @@ MergeWidget::onFileSelectionChanged() {
 
 void
 MergeWidget::onTrackSelectionChanged() {
-  enableInputControls(m_allInputControls, false);
+  Util::enableWidgets(m_allInputControls, false);
 
   auto selection = ui->tracks->selectionModel()->selection();
   if (selection.isEmpty())
@@ -126,40 +135,33 @@ MergeWidget::onTrackSelectionChanged() {
 
   if (1 < selection.size()) {
     setInputControlValues(nullptr);
-    enableInputControls(m_allInputControls, true);
+    Util::enableWidgets(m_allInputControls, true);
     return;
   }
 
-  enableInputControls(m_typeIndependantControls, true);
+  Util::enableWidgets(m_typeIndependantControls, true);
 
-  auto idxs = selection.at(0).indexes();
-  if (idxs.isEmpty() || !idxs.at(0).isValid())
+  auto idxs = selection[0].indexes();
+  if (idxs.isEmpty() || !idxs[0].isValid())
     return;
 
-  auto track = static_cast<Track *>(idxs.at(0).internalPointer());
+  auto track = m_tracksModel->fromIndex(idxs[0]);
   if (!track)
     return;
 
   setInputControlValues(track);
 
   if (track->isAudio())
-    enableInputControls(m_audioControls, true);
+    Util::enableWidgets(m_audioControls, true);
 
   else if (track->isVideo())
-    enableInputControls(m_videoControls, true);
+    Util::enableWidgets(m_videoControls, true);
 
   else if (track->isSubtitles())
-    enableInputControls(m_subtitleControls, true);
+    Util::enableWidgets(m_subtitleControls, true);
 
   else if (track->isChapters())
-    enableInputControls(m_chapterControls, true);
-}
-
-void
-MergeWidget::enableInputControls(QList<QWidget *> const &controls,
-                                 bool enable) {
-  for (auto &control : controls)
-    control->setEnabled(enable);
+    Util::enableWidgets(m_chapterControls, true);
 }
 
 void
@@ -222,10 +224,37 @@ MergeWidget::setInputControlValues(Track *track) {
 
   ui->setAspectRatio->setChecked(        track->m_setAspectRatio);
   ui->setDisplayWidthHeight->setChecked(!track->m_setAspectRatio);
+  ui->fixBitstreamTimingInfo->setChecked(track->m_fixBitstreamTimingInfo);
 
   m_currentlySettingInputControlValues = false;
 }
 
+QList<SourceFile *>
+MergeWidget::selectedSourceFiles()
+  const {
+  auto sourceFiles = QList<SourceFile *>{};
+  Util::withSelectedIndexes(ui->files, [&sourceFiles, this](QModelIndex const &idx) {
+      auto sourceFile = m_filesModel->fromIndex(idx);
+      if (sourceFile)
+        sourceFiles << sourceFile;
+    });
+
+  return sourceFiles;
+}
+
+QList<Track *>
+MergeWidget::selectedTracks()
+  const {
+  auto tracks = QList<Track *>{};
+  Util::withSelectedIndexes(ui->tracks, [&tracks, this](QModelIndex const &idx) {
+      auto track = m_tracksModel->fromIndex(idx);
+      if (track)
+        tracks << track;
+    });
+
+  return tracks;
+}
+
 void
 MergeWidget::withSelectedTracks(std::function<void(Track *)> code,
                                 bool notIfAppending,
@@ -233,8 +262,8 @@ MergeWidget::withSelectedTracks(std::function<void(Track *)> code,
   if (m_currentlySettingInputControlValues)
     return;
 
-  auto selection = ui->tracks->selectionModel()->selection();
-  if (selection.isEmpty())
+  auto tracks = selectedTracks();
+  if (tracks.isEmpty())
     return;
 
   if (!widget)
@@ -246,13 +275,8 @@ MergeWidget::withSelectedTracks(std::function<void(Track *)> code,
   bool withChapters  = m_chapterControls.contains(widget);
   bool withAll       = m_typeIndependantControls.contains(widget);
 
-  for (auto &indexRange : selection) {
-    auto idxs = indexRange.indexes();
-    if (idxs.isEmpty() || !idxs.at(0).isValid())
-      continue;
-
-    auto track = static_cast<Track *>(idxs.at(0).internalPointer());
-    if (!track || (track->m_appendedTo && notIfAppending))
+  for (auto &track : tracks) {
+    if (track->m_appendedTo && notIfAppending)
       continue;
 
     if (   withAll
@@ -358,6 +382,11 @@ MergeWidget::onBrowseTimecodes() {
     withSelectedTracks([&](Track *track) { track->m_timecodes = fileName; });
 }
 
+void
+MergeWidget::onFixBitstreamTimingInfoChanged(bool newValue) {
+  withSelectedTracks([&](Track *track) { track->m_fixBitstreamTimingInfo = newValue; });
+}
+
 void
 MergeWidget::onBrowseTrackTags() {
   auto fileName = getOpenFileName(QY("Select tags file"), QY("XML files") + Q(" (*.xml)"), ui->trackTags);
@@ -446,6 +475,12 @@ MergeWidget::onAddFiles() {
 
 void
 MergeWidget::onAppendFiles() {
+  // auto selectedFile = m_filesModel->fromIndex(selectedSourceFile());
+  // if (selectedFile && !selectedFile->m_tracks.size()) {
+  //   QMessageBox::critical(this, QY("Unable to append files"), QY("You cannot append tracks or add additional parts to files that contain tracks."));
+  //   return;
+  // }
+
   addOrAppendFiles(true);
 }
 
@@ -462,6 +497,9 @@ MergeWidget::addOrAppendFiles(bool append) {
       identifiedFiles << identifier.file();
   }
 
+  if (!append)
+    PlaylistScanner{this}.checkAddingPlaylists(identifiedFiles);
+
   if (identifiedFiles.isEmpty())
     return;
 
@@ -476,6 +514,7 @@ MergeWidget::selectFilesToAdd(QString const &title) {
   dlg.setFileMode(QFileDialog::ExistingFiles);
   dlg.setDirectory(Settings::get().m_lastOpenDir);
   dlg.setWindowTitle(title);
+  dlg.setOptions(QFileDialog::HideNameFilterDetails);
 
   if (!dlg.exec())
     return QStringList{};
@@ -487,7 +526,25 @@ MergeWidget::selectFilesToAdd(QString const &title) {
 
 void
 MergeWidget::onAddAdditionalParts() {
-  m_filesModel->addAdditionalParts(selectedSourceFile(), selectFilesToAdd(QY("Add media files as additional parts")));
+  auto currentIdx = selectedSourceFile();
+  auto sourceFile = m_filesModel->fromIndex(currentIdx);
+  if (sourceFile && !sourceFile->m_tracks.size()) {
+    QMessageBox::critical(this, QY("Unable to append files"), QY("You cannot append tracks or add additional parts to files that contain tracks."));
+    return;
+  }
+
+  m_filesModel->addAdditionalParts(currentIdx, selectFilesToAdd(QY("Add media files as additional parts")));
+}
+
+void
+MergeWidget::onRemoveFiles() {
+  auto selectedFiles = selectedSourceFiles();
+  if (selectedFiles.isEmpty())
+    return;
+
+  m_filesModel->removeFiles(selectedFiles);
+
+  reinitFilesTracksControls();
 }
 
 void
@@ -495,8 +552,10 @@ MergeWidget::onRemoveAllFiles() {
   if (m_config.m_files.isEmpty())
     return;
 
-  m_filesModel->clear();
-  m_tracksModel->clear();
+  m_filesModel->removeRows(0, m_filesModel->rowCount());
+  m_tracksModel->removeRows(0, m_tracksModel->rowCount());
+  m_config.m_files.clear();
+  m_config.m_tracks.clear();
 
   reinitFilesTracksControls();
 }
@@ -513,13 +572,13 @@ MergeWidget::reinitFilesTracksControls() {
 void
 MergeWidget::resizeFilesColumnsToContents()
   const {
-  resizeViewColumnsToContents(ui->files);
+  Util::resizeViewColumnsToContents(ui->files);
 }
 
 void
 MergeWidget::resizeTracksColumnsToContents()
   const {
-  resizeViewColumnsToContents(ui->tracks);
+  Util::resizeViewColumnsToContents(ui->tracks);
 }
 
 void
@@ -552,10 +611,6 @@ MergeWidget::retranslateInputUI() {
 QModelIndex
 MergeWidget::selectedSourceFile()
   const {
-  auto selection = ui->files->selectionModel()->selection();
-  if (selection.isEmpty())
-    return QModelIndex{};
-
-  auto idxs = selection.at(0).indexes();
-  return idxs.isEmpty() ? QModelIndex{} : idxs.at(0);
+  auto idx = ui->files->selectionModel()->currentIndex();
+  return m_filesModel->index(idx.row(), 0, idx.parent());
 }
